import gql from "graphql-tag";
import { useMutation as baseMutation } from "@vue/apollo-composable";
import { toPascalCase } from "../utils/string/toPascalCase";
import { toCamelCase } from "../utils/string/toCamelCase";
import { isString } from "../utils/string/isString";
import { ApolloError } from "@apollo/client/errors";
import { ValidationError } from "./errors/ValidationError";
export const useMutation = (name, options) => {
    const resp = options != undefined && isString(options.response)
        ? `{ ${options.response} }`
        : "";
    const doc = `mutation ${name}($data: ${toPascalCase(name)}Data!) {
    ${toCamelCase(name)}(data: $data) ${resp} 
  }`;
    const document = gql `
    ${doc}
  `;
    const { loading, error, mutate } = baseMutation(document, {
    // fetchPolicy: "no-cache",
    // refetchQueries: ["name"],
    });
    return {
        isLoading: loading,
        error,
        submit(data) {
            return mutate({ data })
                .then((res) => {
                if (res?.data == null) {
                    return null;
                }
                // @ts-ignore
                return res?.data[name] ?? null;
            })
                .catch((error) => {
                if (error instanceof ApolloError) {
                    const validation = error.graphQLErrors?.filter((i) => i.extensions.category === "validation") ?? [];
                    throw validation.length === 0
                        ? error
                        : new ValidationError(validation.reduce(
                        // @ts-ignore
                        (acc, item) => ({ ...acc, ...item.extensions.validation }), {}), error.message);
                }
                else {
                    throw error;
                }
            });
        },
    };
};
