import { computed, ref, watch } from "vue";
import { request, createBackendError } from "./request";
import { isString } from "../utils/string/isString";
import { handleError } from "./index";
export const useRequest = function (path, options) {
    const isLoading = ref(false);
    const data = ref(null);
    const error = ref(null);
    const currentPath = computed(() => {
        return isString(path) ? path : path();
    });
    const call = (extra) => {
        isLoading.value = true;
        error.value = null;
        return request(currentPath.value, { ...options, ...extra })
            .json()
            .then((json) => {
            data.value = json;
            isLoading.value = false;
            return json;
        })
            .catch((err) => {
            const backendError = createBackendError(err);
            handleError(err);
            isLoading.value = false;
            error.value = backendError;
            if (extra?.throwError !== false) {
                throw backendError;
            }
        });
    };
    if (options?.autoload === true) {
        call({ throwError: false }).catch(console.error);
    }
    watch(currentPath, () => {
        if (options?.autoload === true) {
            call({ throwError: false }).catch(console.error);
        }
    });
    return {
        isLoading,
        data,
        error,
        send: call,
    };
};
