function getStacktrace(error) {
    if (typeof error.stacktrace !== "undefined" ||
        typeof error["opera#sourceloc"] !== "undefined") {
        // probably opera
        const { stacktrace = "" } = error;
        return stacktrace
            .split("\n")
            .filter((line) => line !== "");
    }
    else if (error.stack) {
        // an Error or a plain object that resembles an error
        const { stack = "" } = error;
        return stack.split("\n").filter((line) => line !== "");
    }
    else {
        // probably IE9 :(
        return ["No stacktrace available"];
    }
}
function errorJsonReplacer() {
    const seenValue = [];
    const seenKey = [];
    return (key, value) => {
        if (typeof value === "object" && value !== null) {
            const i = seenValue.indexOf(value);
            if (i !== -1) {
                return `[cyclic value: ${seenKey[i] || "root object"}]`;
            }
            else if (value instanceof Response) {
                return {
                    ok: value.ok,
                    redirected: value.redirected,
                    status: value.status,
                    statusText: value.statusText,
                    url: value.url,
                    type: value.type,
                };
            }
            else if (value instanceof Request) {
                return { method: value.method, url: value.url };
            }
            else {
                seenValue.push(value);
                seenKey.push(key);
            }
        }
        return value;
    };
}
export const openErrorReportWindow = (error) => {
    const winPrint = window.open("", "", "width=800,height=600,toolbar=0");
    if (winPrint == null) {
        return;
    }
    const report = JSON.stringify({
        name: error.name,
        message: error.message,
        error,
        stacktrace: getStacktrace(error),
    }, errorJsonReplacer(), 3);
    // @ts-ignore
    const requestId = error.request?.id ?? "[not set]";
    winPrint.document.write(`
    <title>Error Report</title>
    <style rel="stylesheet">
      .copy-btn {
        padding: .2rem .2rem; background: white; border: 1px solid #666; border-radius: 3px;
        cursor: pointer;
      }
      .copy-btn .copy {
        display: block;
      }
      .copy-btn .check {
        display: none;
      }
      .copy-btn.copied .copy {
        display: none;
      }
      .copy-btn.copied .check {
        display: block;
      }
      .copy-btn:hover {
        background: #e0e0e0;
      }
    </style>
    <body style="padding: 1rem; background: #eee; color: #333">
      <div style="display: flex; justify-content: space-between; align-items: center">
        <div>
          <h3 style="margin: .25rem 0">${error.name}</h3>
          <h1 style="margin: .25rem 0">${error.message}</h1>
          <p>Request ID: ${requestId}</p>
        </div>
        <div>
          ${new Date().toISOString()}
        </div>
      </div>
      <textarea 
        readonly 
        id="report"
        rows="100"
        style="height: calc(100vh - 220px); margin-top: .5rem; resize: none; width: 100%; overflow: auto; padding: .1rem; background: white; color: #333; font-size: 12px; border: 1px solid #ddd; border-radius: .25rem"
      >${report}</textarea>
      <div style="padding: .5rem 0; display: flex; align-items: center">
        <div>
          <button onclick="copy2clipboard('report')" class="copy-btn" id="copy-btn">
            <svg fill="#000000" class="copy" stroke="currentColor" height="18px" width="18px" viewBox="0 0 64 64">
              <g>
                <path d="M53.9791489,9.1429005H50.010849c-0.0826988,0-0.1562004,0.0283995-0.2331009,0.0469999V5.0228   C49.7777481,2.253,47.4731483,0,44.6398468,0h-34.422596C7.3839517,0,5.0793519,2.253,5.0793519,5.0228v46.8432999   c0,2.7697983,2.3045998,5.0228004,5.1378999,5.0228004h6.0367002v2.2678986C16.253952,61.8274002,18.4702511,64,21.1954517,64   h32.783699c2.7252007,0,4.9414978-2.1725998,4.9414978-4.8432007V13.9861002   C58.9206467,11.3155003,56.7043495,9.1429005,53.9791489,9.1429005z M7.1110516,51.8661003V5.0228   c0-1.6487999,1.3938999-2.9909999,3.1062002-2.9909999h34.422596c1.7123032,0,3.1062012,1.3422,3.1062012,2.9909999v46.8432999   c0,1.6487999-1.393898,2.9911003-3.1062012,2.9911003h-34.422596C8.5049515,54.8572006,7.1110516,53.5149002,7.1110516,51.8661003z    M56.8888474,59.1567993c0,1.550602-1.3055,2.8115005-2.9096985,2.8115005h-32.783699   c-1.6042004,0-2.9097996-1.2608986-2.9097996-2.8115005v-2.2678986h26.3541946   c2.8333015,0,5.1379013-2.2530022,5.1379013-5.0228004V11.1275997c0.0769005,0.0186005,0.1504021,0.0469999,0.2331009,0.0469999   h3.9682999c1.6041985,0,2.9096985,1.2609005,2.9096985,2.8115005V59.1567993z"/>
                <path d="M38.6031494,13.2063999H16.253952c-0.5615005,0-1.0159006,0.4542999-1.0159006,1.0158005   c0,0.5615997,0.4544001,1.0158997,1.0159006,1.0158997h22.3491974c0.5615005,0,1.0158997-0.4542999,1.0158997-1.0158997   C39.6190491,13.6606998,39.16465,13.2063999,38.6031494,13.2063999z"/>
                <path d="M38.6031494,21.3334007H16.253952c-0.5615005,0-1.0159006,0.4542999-1.0159006,1.0157986   c0,0.5615005,0.4544001,1.0159016,1.0159006,1.0159016h22.3491974c0.5615005,0,1.0158997-0.454401,1.0158997-1.0159016   C39.6190491,21.7877007,39.16465,21.3334007,38.6031494,21.3334007z"/>
                <path d="M38.6031494,29.4603004H16.253952c-0.5615005,0-1.0159006,0.4543991-1.0159006,1.0158997   s0.4544001,1.0158997,1.0159006,1.0158997h22.3491974c0.5615005,0,1.0158997-0.4543991,1.0158997-1.0158997   S39.16465,29.4603004,38.6031494,29.4603004z"/>
                <path d="M28.4444485,37.5872993H16.253952c-0.5615005,0-1.0159006,0.4543991-1.0159006,1.0158997   s0.4544001,1.0158997,1.0159006,1.0158997h12.1904964c0.5615025,0,1.0158005-0.4543991,1.0158005-1.0158997   S29.0059509,37.5872993,28.4444485,37.5872993z"/>
              </g>
            </svg>
            <svg fill="#000000" class="check" stroke="currentColor" height="18px" width="18px" viewBox="0 0 512 512">
              <g>
                <path d="M474.045,173.813c-4.201,1.371-6.494,5.888-5.123,10.088c7.571,23.199,11.411,47.457,11.411,72.1   c0,62.014-24.149,120.315-68,164.166s-102.153,68-164.167,68s-120.316-24.149-164.167-68S16,318.014,16,256   S40.149,135.684,84,91.833s102.153-68,164.167-68c32.889,0,64.668,6.734,94.455,20.017c28.781,12.834,54.287,31.108,75.81,54.315   c3.004,3.239,8.066,3.431,11.306,0.425c3.24-3.004,3.43-8.065,0.426-11.306c-23-24.799-50.26-44.328-81.024-58.047   C317.287,15.035,283.316,7.833,248.167,7.833c-66.288,0-128.608,25.813-175.48,72.687C25.814,127.392,0,189.712,0,256   c0,66.287,25.814,128.607,72.687,175.479c46.872,46.873,109.192,72.687,175.48,72.687s128.608-25.813,175.48-72.687   c46.873-46.872,72.687-109.192,72.687-175.479c0-26.332-4.105-52.26-12.201-77.064   C482.762,174.736,478.245,172.445,474.045,173.813z"/>
                <path d="M504.969,83.262c-4.532-4.538-10.563-7.037-16.98-7.037s-12.448,2.499-16.978,7.034l-7.161,7.161   c-3.124,3.124-3.124,8.189,0,11.313c3.124,3.123,8.19,3.124,11.314-0.001l7.164-7.164c1.51-1.512,3.52-2.344,5.66-2.344   s4.15,0.832,5.664,2.348c1.514,1.514,2.348,3.524,2.348,5.663s-0.834,4.149-2.348,5.663L217.802,381.75   c-1.51,1.512-3.52,2.344-5.66,2.344s-4.15-0.832-5.664-2.348L98.747,274.015c-1.514-1.514-2.348-3.524-2.348-5.663   c0-2.138,0.834-4.149,2.351-5.667c1.51-1.512,3.52-2.344,5.66-2.344s4.15,0.832,5.664,2.348l96.411,96.411   c1.5,1.5,3.535,2.343,5.657,2.343s4.157-0.843,5.657-2.343l234.849-234.849c3.125-3.125,3.125-8.189,0-11.314   c-3.124-3.123-8.189-3.123-11.313,0L212.142,342.129l-90.75-90.751c-4.533-4.538-10.563-7.037-16.98-7.037   s-12.448,2.499-16.978,7.034c-4.536,4.536-7.034,10.565-7.034,16.977c0,6.412,2.498,12.441,7.034,16.978l107.728,107.728   c4.532,4.538,10.563,7.037,16.98,7.037c6.417,0,12.448-2.499,16.977-7.033l275.847-275.848c4.536-4.536,7.034-10.565,7.034-16.978   S509.502,87.794,504.969,83.262z"/>
              </g>
            </svg>
          </button>
        </div>
        <div style="margin-left: .5rem">Передайте цю інформацію розробнику або напишіть в Telegram-бот</div>
      </div>
    </body>
    <script type="text/javascript">
      let timeout;
      function copy2clipboard(id) {
        let copyText = document.getElementById(id);
        navigator.clipboard.writeText(copyText.value);
        document.getElementById('copy-btn').classList.add('copied');
        clearTimeout(timeout);
        timeout = setTimeout(function() {
          document.getElementById('copy-btn').classList.remove('copied');
        }, 5000);
      }
    </script>
  `);
};
