import { TYPE, useToast } from "vue-toastification";
import { isString } from "../utils/string/isString";
import { ValidationError } from "../backend/errors/ValidationError";
import { ClientError } from "../backend/errors/ClientError";
import { RedirectionError } from "../backend/errors/RedirectionError";
import { openErrorReportWindow } from "./openErrorReportWindow";
const SUCCESS_CONFIG = {
    timeout: 3000,
};
const INFO_CONFIG = {
    timeout: 4000,
};
const WARN_CONFIG = {
    timeout: 5000,
};
const ERROR_CONFIG = {
    timeout: 6000,
};
export const useNotify = (options) => {
    const toast = useToast();
    return {
        ok(message) {
            toast.success(message, SUCCESS_CONFIG);
        },
        info(message) {
            toast.info(message, INFO_CONFIG);
        },
        warn(message) {
            toast.warning(message, WARN_CONFIG);
        },
        error(error) {
            if (isString(error)) {
                return toast.error(error, ERROR_CONFIG);
            }
            if (error instanceof ValidationError) {
                return Object.keys(error.errors)
                    .map((field) => error.errors[field].join("\n"))
                    .forEach((field) => toast.error(field, ERROR_CONFIG));
            }
            if (error instanceof RedirectionError) {
                return toast.info(error.message, ERROR_CONFIG);
            }
            if (error instanceof ClientError) {
                return toast.warning(error.message, ERROR_CONFIG);
            }
            return toast({
                component: options?.errorComponent ?? undefined,
                props: {
                    message: error.message,
                },
                listeners: {
                    view: () => openErrorReportWindow(error),
                },
            }, {
                type: TYPE.ERROR,
                timeout: 30000,
                closeOnClick: false,
            });
        },
        clear() {
            toast.clear();
        },
    };
};
